.module Collider_System

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;      INCLUDES        ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "entity.inc"
.include "HUD.inc"
.include "IA.inc"
.include "sprites.inc"
.include "music.inc"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl sys_ai_update_one_entity
.globl sys_render_borrar
.globl man_game_entity_destroy
.globl temp_roja
.globl temp_azul
.globl temp_verde

.globl show_victory_screen
.globl enemies_onscreen

.globl init_object
.globl man_game_create_template_entity

.globl cpct_drawSolidBox_asm
.globl cpct_drawSpriteMaskedAlignedTable_asm
.globl _g_masktable

.globl hat_timer
.globl player_damaged

.globl wave_finished

contador_colision: .db 1

powerup_metr_timer:: .db 0
powerup_multi_timer:: .db 0
powerup_metr:: .db 0
powerup_multi:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Collisions_Update_One_Entity'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Primera entidad
;;        (IY) Segunda entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comrpueba si las dos entidades pueden 
;; colisionar entre si, y si es asi
;; comprobamos si los bounding box se
;; cumple. 
;;
;;
sys_collision_update_entities::
    ld b, Ent_X(ix)
    srl b
    ld c, Ent_W(ix)
    ld d, Ent_X(iy)
    srl d
    ld e, Ent_W(iy)

    ld a, Ent_AIPREV1(iy)
    cp #<sys_ia_seguir_jugador
    jp nz,no_collision_modifier_coyote_X
        ld a, Ent_DIRECCION(iy)
        cp #0
        jp z,anyadir_modificador_zorro
        cp #4
        jp z,anyadir_modificador_zorro
        jp no_collision_modifier_coyote_X
        anyadir_modificador_zorro:
            ld a,#1
            add a,d
            ld d,a
            ld a,#-2
            add a,e
            ld e,a
        jp stop_modifier
    no_collision_modifier_coyote_X:
    cp #<sys_ia_sniper
    jp z,modificar_bandido
    cp #<sys_ia_sniper_en_posicion
    jp z,modificar_bandido
    cp #<sys_ia_patrol
    jp z,modificar_bandido
    jp stop_modifier
    modificar_bandido:
        dec d
        dec d
        inc e
        inc e
        inc e
        inc e
    stop_modifier:

    ld hl,#colision_x
    jp check_collision
    colision_x:

    ld b, Ent_Y(ix)
    ld c, Ent_H(ix)
    ld d, Ent_Y(iy)
    ld e, Ent_H(iy)
    ld hl,#colision_y
    jp check_collision
    colision_y:

    ld   a, Ent_TYPE(ix)
    cp   #e_type_soldier
    jp   nz, no_follow
        ld a,(hat_timer) 
        cp #0
        jp nz,saltar_desaparecer
            call man_game_entity_destroy
            call decrease_enemies
        saltar_desaparecer:
        jp   no_hit_object
    no_follow:
    ld   a, Ent_TYPE(ix)
    cp   #e_type_shot
    jp   nz, no_hit_shot  
        call man_game_entity_destroy
        jp   no_hit_object
    no_hit_shot:
    ld a, Ent_TYPE(ix)
    cp   #e_type_object
    jp   nz, no_hit_object
            ld a, Ent_AI_COUNT(ix)
            cp #1
            jr nz, no_metra
                ld (powerup_metr),a
                ld bc,#_vaso_2
                call draw_powerup
                ld a,#255
                ld (powerup_metr_timer),a
                ld a,#0
                ld Ent_COOLDOWN(iy),a
                ld a,#10
                ld Ent_COOLDOWN_O(iy),a
                jp fin_power
            no_metra:
            cp #2
            jr nz, no_multi
                ld (powerup_multi),a
                ld bc,#_vaso_1
                call draw_powerup
                ld a,#255
                ld (powerup_multi_timer),a
                jp fin_power
            no_multi:
            cp #3
            jr nz, no_vida
                ld   b, #1
                call man_HUD_add_life  
                cpctm_setBorder_asm HW_GREEN        ; Cambiamos el color de los bordes
                ld a, #20
                ld (hat_timer),a
            no_vida:
            fin_power:

            call man_game_entity_destroy

            ld   b, #5
            call man_HUD_add_unidad

            call sys_music_play_powerup

            jp   saltar_limpiado
    no_hit_object:

    ld   a, Ent_TYPE(iy)
    cp   #e_type_player
    jp   nz, no_hit_player
        ld a,(hat_timer) 
        cp #0
        jp nz,saltar_danyo
            call sys_music_play_playerHit
            ld   b, #0xFF 
            call man_HUD_add_life  
            cpctm_setBorder_asm HW_RED        ; Cambiamos el color de los bordes
            ld a, #20
            ld (hat_timer),a
            ld a,#1
            ld (player_damaged),a
        saltar_danyo:
    jp   saltar_limpiado
    no_hit_player:
    ld   a, Ent_TYPE(iy)
    cp   #e_type_soldier
    jp   nz, no_hit_soldier
            call sys_music_play_hit
            ld Ent_AI1(iy),#<sys_ia_morir
            ld Ent_AI2(iy),#>sys_ia_morir
            ld Ent_AI_COUNT(iy),#20
            ld Ent_AI_COUNT_O(iy),#20
            ld a,#e_cmp_render
            or #e_cmp_ai
            ld Ent_COMPONENTS(iy),a
            jp   saltar_limpiado
    no_hit_soldier:
    saltar_limpiado:
    no_colisiona:                       ;; Saltamos a este punto si no colisionamos
ret
check_collision:
    inc b
    dec c
    dec c
    inc d
    dec e
    dec e

    ld a,d                  
    add e
    cp b
    jp c, no_colisiona
    jp z, no_colisiona
    ld a,b                  
    add c
    cp d
    jp c, no_colisiona
    jp z, no_colisiona
    jp (hl)

decrease_enemies::
    ld a, (enemies_onscreen)
    dec a
    ld (enemies_onscreen), a
ret

;first word in bc, second in de
;destination in hl
compare_word:
    ld   a, d
    cp   c
    jp  nz, fin_compare_word
    ld   a,e
    cp   b
    jp  nz, fin_compare_word
    pop af
    jp  (hl)
    fin_compare_word:
ret
sys_collision_enemy_point::
    push ix
    ld  b, Ent_AIPREV1(ix)
    ld  c, Ent_AIPREV2(ix)

    ; Compruebo patrullas
    ld de, #sys_ia_patrol
    ld hl, #es_patrulla
    call compare_word

    ; Compruebo sniper
    ld de, #sys_ia_sniper_en_posicion
    ld hl, #es_sniper
    call compare_word

    ;Compruebo el seguidor
    ld de, #sys_ia_seguir_jugador
    ld hl, #es_seguidor
    call compare_word

    ;Compruebo el bouncer
    ld de, #sys_ia_bouncer
    ld hl, #es_bouncer
    call compare_word
    jp fin_points

    es_bouncer:
    ld   b, #2
    call man_HUD_add_diez
    ld   b, #1
    call man_HUD_add_cien
    jp fin_points

    es_patrulla:
    ld   b, #5
    call man_HUD_add_diez
    jp   fin_points

    es_sniper:
    ld   b, #1
    call man_HUD_add_cien
    jp   fin_points

    es_seguidor:
    ld   b, #5
    call man_HUD_add_diez
    ld   b, #1
    call man_HUD_add_cien
    fin_points::
    pop ix
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Collision_Update'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Llama a la funcion de entidad que 
;; recorre todo el array en busca de dos
;; entidades con la componente de colision
;; y luego llama a la funcion en (HL)
;;
sys_collision_update::
    call man_entity_forall_pairs_matching
ret

generar_objeto::
    ld b,Ent_X(ix) 
    ld a,Ent_Y(ix)
    add a, #10
    ld c,a
    push ix
    push bc

    ld  a, (temp_azul)
    dec a
    ld  (temp_azul), a
    jp  z, spawn_metra 

    ld  a, (temp_verde)
    dec a
    ld  (temp_verde), a
    jp  z, spawn_multi

    ld  a, (temp_roja)
    dec a
    ld  (temp_roja), a
    jp  z, spawn_vida
   
    pop bc
    pop ix
    ret

    spawn_vida:
        ld hl, #init_object
        call man_game_create_template_entity
        pop bc
        ld Ent_X(ix),b
        ld Ent_Y(ix),c

        ld a, #3
        ld Ent_AI_COUNT(ix), a
        ld a, #4
        ld (temp_roja), a
    jr final_generar_object

    spawn_multi:
        ld hl, #init_object
        call man_game_create_template_entity
        pop bc
        ld Ent_X(ix),b
        ld Ent_Y(ix),c

        ld a, #2
        ld Ent_AI_COUNT(ix), a
        ld hl, #sprites_verde
        ld Ent_SPRITE1(ix), l
        ld Ent_SPRITE2(ix), h
        ld a, #2
        ld (temp_verde), a
    jr final_generar_object

    spawn_metra:
        ld hl, #init_object
        call man_game_create_template_entity
        pop bc
        ld Ent_X(ix),b
        ld Ent_Y(ix),c

        ld a, #1
        ld Ent_AI_COUNT(ix), a
        ld hl, #sprites_azul
        ld Ent_SPRITE1(ix), l
        ld Ent_SPRITE2(ix), h
        ld a, #3
        ld (temp_azul), a
    
    final_generar_object:
    pop ix
ret

;requiere en bc sprite de powerup
draw_powerup::
    ld de,#0
    ld a, c
    cp #<_vaso_1
    jp z, es_vaso1
        ld de,#4
    es_vaso1:

    push ix
    push bc
    push de
    ld hl,#0xe748
    call draw_powerup_sprite
    pop de
    pop bc
    ld hl,#0xa748
    call draw_powerup_sprite
    pop ix
ret
draw_powerup_sprite:
    add hl,de
    ld d,h
    ld e,l
    ld l, #SPR_OBJECT_W
    ld h, #SPR_OBJECT_H
    push hl
    pop ix
    ld h, #>_g_masktable
    ld l, #<_g_masktable
        ;(2B BC) psprite	Source Sprite Pointer
        ;(2B DE) pvideomem	Destination video memory pointer
        ;(1B IXL) width	Sprite Width in bytes (>0) (Beware, not in pixels!)
        ;(1B IXH) height	Sprite Height in bytes (>0)
        ;(2B HL) pmasktable0	Pointer to an Aligned Mask Table for transparencies with palette index 0
    call cpct_drawSpriteMaskedAlignedTable_asm
ret

;requiere #4 en hl si metr
erase_powerup::
    push hl
        ;(2B DE) memory	Video memory pointer to the upper left box corner byte
        ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
        ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
        ;(1B B ) height	Box height in bytes (>0)
    ld de,#0xe748
    call erase_powerup_box
    pop hl
    ld de,#0xa748
    call erase_powerup_box
ret
erase_powerup_box:
    add hl,de
    ld d,h
    ld e,l
    ld a,#0x30
    ld c, #SPR_OBJECT_W
    ld b, #SPR_OBJECT_H
    call cpct_drawSolidBox_asm
ret